from tinynetrc import Netrc
import sys
import dong

def get_credential_or_exit(message="Can't find any credential information, please login first."):
    netrc = Netrc()
    login = netrc[dong.SERVER_NAME]['login']
    password = netrc[dong.SERVER_NAME]['password']
    if login is None or password is None:
        print(message)
        sys.exit(1)

    return (login, password)



import requests
import dong
import logging

def post(api, **kwargs):
    url = "{}/{}".format(dong.SERVER_IP, api)
    logging.info("post:{}".format(url))
    r = requests.post(url, **kwargs)
    return r

def get(api, **kwargs):
    url = "{}/{}".format(dong.SERVER_IP, api)
    logging.info("get:{}".format(url))
    r = requests.get(url, **kwargs)
    return r



from tinynetrc import Netrc

import dong

def login_required(message="Can't find any credential information, please login first."):

    def decorator(fun):

        def wrapper(*args, **kwargs):
            netrc = Netrc()
            login = netrc[dong.SERVER_NAME]['login']
            password = netrc[dong.SERVER_NAME]['password']

            if login is None or password is None:
                print(message)
            else:
                fun(*args, **kwargs)

        return wrapper

    return decorator



import click
import dong
import dong.httpclient as httpclient
#import requests

@click.command()
def command():
   """[Not released yet] storage integration"""
   click.echo('command \"data\" Not released in pre-α')
   return None

# def _copy(src, dst):
#    click.echo('copy %s to folder %s ...' % (src, dst))
#    files = {'file': open(src, 'rb')}
#    r = httpclient.post('api/v1/copy/', json = {'dst_name': dst}, files = files)
#    if r.status_code == 200:
#       click.echo('data [%s] URL lg://%s' % (src,dst))
#    else:
#       print('ERROR,[internet connection, sever error, ...]')
#       return None

# @click.group()
# def command():
#    """storage integration"""
#    pass

# @command.command(help="Copy local file to bucket")
# @click.argument('src', nargs=-1, required = True)
# @click.argument('dst', nargs=1, required = True)
# def cp(src, dst):
#    _copy(src, dst)

# # @command.command(help="Make a bucket")
# # @click.argument('dst', nargs=1, required = True)
# # def mb(dst):
# #    click.echo("mb")

# @command.command(help="Remove file on cloud")
# @click.argument('dst', nargs=1, required = True)
# def rm(dst):
#    click.echo("remove")

# @command.command(help="List dst/file on cloud")
# def ls():
#    click.echo("list")#顯示bucket的list
#  No newline at end of file



import os
import click

from dong import utils
from dong import templates


template_help = """module name for to be generated {} module from the template."""

@click.group(invoke_without_command=True)
@click.option('--data-module', default=None, help=template_help.format('data'))
@click.option('--model-init-module', default=None, help=template_help.format('model init'))
@click.option('--model-serializer-module', default=None, help=template_help.format('model serializer'))
@click.option('--model-train-module', default=None, help=template_help.format('model train'))
@click.option('--model-module', default=None, help=template_help.format('model'))
@click.option('--config-module', default=None, help=template_help.format('config'))
@click.option('--service-module', default=None, help=template_help.format('service'))
@click.option('--tune-module', default=None, help=template_help.format('tune'))
# handle model compose later
@click.pass_context
def command(ctx,
            data_module,
            model_init_module,
            model_serializer_module,
            model_train_module,
            model_module,
            config_module,
            service_module,
            tune_module):
    """Generate module files from template, if not provided, the corresponding module won't be generated."""

    modules = {
        'data': data_module,
        'model/init': model_init_module,
        'model/serializer': model_serializer_module,
        'model/train': model_train_module,
        'model': model_module,
        'config': config_module,
        'service': service_module,
        'tune': tune_module,
    }

    if all( modules[key] is None for key in modules.keys() ):
        click.echo(ctx.get_help(), color=ctx.color)
        ctx.exit(1)
        return

    # NOTICE: should do exception handling when there is no setup.py
    project_name = utils.get_project_name('./setup.py')

    
    for name in modules.keys():
        if modules[name] is None:
            continue

        module_dir = './' + project_name + '/' + name

        if not os.path.isdir(module_dir):
            utils.eprint('Directory ' + name + "/ doesn't exist. Are you inside a dong project?")
            continue

        # by modules[name] we get the name of the module to be generated
        utils.create_file(os.path.join(module_dir, modules[name] + '.py'),
                          templates.make_template(name,
                                                  { 'class_name': modules[name].title() + name.title().replace('/', '')}))



import os

#__version__ = "0.1"

PROG_NAME = "dong"

try:
    os.environ['DONG_DEBUG']
except KeyError:
    SERVER_IP = 'https://dongcloud.libgirl.com'
else:
    SERVER_IP = 'http://127.0.0.1:8000'

SERVER_NAME = 'api.libgirl.com'




import os
import dong.templates as templates
import dong.utils as utils

def define_structure(opts):
    s = {
        'setup.py': templates.make_template('setup_py', opts),
        opts['package']: {
            '__init__.py': templates.make_template('__init__', opts),
            'data': {
                '__init__.py': '',
                'default.py': templates.make_template('data', {**opts, **{'class_name': 'DefaultData'}}),
            },
            'model': {
                '__init__.py': '',
                'default.py': templates.make_template('model', {**opts, **{'class_name': 'DefaultModel'}}),
                'init': {
                    '__init__.py': '',
                    'default.py': templates.make_template('model/init', {**opts, **{'class_name': 'DefaultModelInit'}}),
                },
                'serializer': {
                    '__init__.py': '',
                    'default.py': templates.make_template('model/serializer', opts),
                },
                'train': {
                    '__init__.py': '',
                    'default.py': templates.make_template('model/train', opts),
                },
            },
            'config': {
                '__init__.py': '',
                'default.py': templates.make_template('config', opts),
            },
            'tune': {
                '__init__.py': '',
                'default.py': templates.make_template('tune', {**opts, **{'class_name': 'DefaultTune'}}),
            },
            'service': {
                '__init__.py': '',
                'default.py': templates.make_template('service', {**opts, **{'class_name': 'DefaultService'}}),
            },
        },
    }

    return s

def create_structure(struct, opts, prefix=None):
    if prefix is None:
        prefix = os.getcwd()

    changed = {}
    for name, content in struct.items():
        if isinstance(content, str):
            utils.create_file(os.path.join(prefix, name), content)
            changed[name] = content
        elif isinstance(content, dict):
            utils.create_directory(os.path.join(prefix, name))
            changed[name] = create_structure(struct[name], opts, prefix=os.path.join(prefix, name))
        elif content is None:
            pass

    return changed



import os, sys

import click

cmd_folder = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                          'commands'))

class DongCLI(click.MultiCommand):

    def list_commands(self, ctx):
        rv = []
        for filename in os.listdir(cmd_folder):
            if filename.endswith('.py') and \
               filename.startswith('cmd_'):
                rv.append(filename[4:-3])
        rv.sort()
        return rv

    def get_command(self, ctx, name):
        try:
            if sys.version_info[0] == 2:
                name = name.encode('ascii', 'replace')
            mod = __import__('dong.commands.cmd_' + name,
                             None, None, ['command'])
        except ImportError:
            return
        return mod.command


@click.command(cls=DongCLI)
def main():
    """Universal Command Line Interface for Libgirl AI Platform"""
    pass



import click

from tinynetrc import Netrc

import dong
import dong.httpclient as httpclient

def _save(email, password):
        netrc = Netrc()
        netrc[dong.SERVER_NAME] = {
            'login': email,
            'password': password
        }
        netrc.save()


def _login(username, password):
    r = httpclient.post('api/v1/login/', json={'email': username, 'password': password})
    if r.status_code == 200:
        print("Login success!")

        password = r.json()['token']
        _save(
            email=username,
            password=password
        )
    else:
        print("Login failed")


@click.command(help='Login with your credentials.')
@click.option('-u', '--username', prompt=True)
@click.option('-p', '--password', prompt=True, hide_input=True)
def command(username, password):
    _login(username, password)



from __future__ import print_function
import os
import subprocess
import sys

def create_directory(path):
    try:
        os.mkdir(path)
    except OSError:
        # TODO: handle exception someday
        raise


def create_file(path, content):
    with open(path, "w") as f:
        f.write(content)

            
def get_project_name(setup_py_path):
    args = ['python', setup_py_path, '--name']
    p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    (stdout, stderr) = p.communicate()
    
    return stdout.decode('UTF-8')[0:-1]


def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)



import click
import os
import dong
import pkg_resources 

version = pkg_resources.require("dong")[0].version
@click.command(help='Print version and exit.', add_help_option=False)
def command():
    click.secho("dong " + version, fg='green')


import contextlib
import os
import shutil
import subprocess
import sys
import tempfile
import time
import json

import click
import dong
from dong import auth
from dong import net
from dong import utils
from dong.decorators import login_required
import dong.httpclient as httpclient

usage_tip = "\n[Usage] Status check:\ndong train status -j {}\n"

def is_training_succeeded(job):
    r = httpclient.get('api/v1/train/{}/'.format(job))
    result = r.json()
    if result['status'] == 'Succeeded':
        return True
    else:
        return False


def _wait_for_completion(job):
    count = 0
    while is_training_succeeded(job) is False:
        print('.', end='', flush=True)
        time.sleep(1)
        count += 1

    print('👍')


@contextlib.contextmanager
def _tempdir(change_to=False):
    temp_dir = tempfile.mkdtemp()
    cur_dir = None
    if change_to:
        cur_dir = os.getcwd()
        os.chdir(temp_dir)

    yield temp_dir

    if cur_dir is not None:
        os.chdir(cur_dir)

    _rmtree(temp_dir)


def _copytree(source_dir, temp_dir):
    dest_dir = os.path.join(temp_dir, 'dest')
    shutil.copytree(source_dir, dest_dir)
    return dest_dir


def _rmtree(path):
    shutil.rmtree(path)


def _exec(setup_py_path, setup_py_args, env=None):
    args = ['python', setup_py_path] + setup_py_args
    p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env)
    (stdout, stderr) = p.communicate()

    return p.returncode


@contextlib.contextmanager
def _cd(working_dir):
    curdir = os.getcwd()
    try:
        os.chdir(working_dir)
        yield
    finally:
        os.chdir(curdir)


def _run_setuptools(package_root, setup_py_path, output_dir):
    with _cd(package_root):
        sdist_args = ['sdist', '--dist-dir', output_dir]
        setup_py_args = sdist_args

        _exec(setup_py_path, setup_py_args)

        local_paths = [os.path.join(output_dir, rel_file)
                       for rel_file in os.listdir(output_dir)]
        return local_paths


def _upload_package(password, module_name, path_to_file, message, args):
    file_name = os.path.basename(path_to_file)

    multipart_form_data = {
        'file': (file_name, open(path_to_file, 'rb')),
        'module_name': (None, module_name),
        'message': (None, message),
        'args': (None, args)
    }
    headers = net.authorization_headers(password)
    r = httpclient.post('api/v1/train/', files=multipart_form_data, headers=headers)
    result = r.json()

    if 'name' in result:
        return r.json()['name']
    else:
        return None


def _build_packages(package_path, output_dir):
    package_path = os.path.abspath(package_path)
    package_root = os.path.dirname(package_path)

    with _tempdir(True) as working_dir:
        package_root = _copytree(package_root, working_dir)

        setup_py_path = os.path.join(package_root, 'setup.py')
        return _run_setuptools(package_root, setup_py_path, output_dir)


def _execute(password, package_path, module_name, message, args):
    with _tempdir(False) as working_dir:
        print("Building package...")
        paths = _build_packages(package_path,
                                os.path.join(working_dir, 'output'))
        path_to_file = paths[0]
        print("Uploading package & generating Job-name...")
        try:
            job_name = _upload_package(password, module_name, path_to_file, message, args)
        except Exception as e:
            job_name = None

        if job_name is None:
            print("Can't upload package, stopped.")
            return
        
        click.echo(
            "\n"
            + "New Job-name: "
            + click.style(" {} ".format(job_name), reverse=True)
            + "\n"
            + usage_tip.format(job_name)
            )


@click.group()
def command():
    """Training job."""
    pass


@command.command()
@click.option('--package-path', default=None)
@click.option('-m', '--message', prompt='Training message')
@click.argument('args', nargs=-1)
@click.pass_context
def exec(ctx, package_path, message,args):
    """Execute training job."""

    module_name = utils.get_project_name('./setup.py' if package_path is None else os.path.join(os.path.dirname(package_path),
                                                                                                'setup.py'))
    if module_name == '' or len(module_name.split('/')) > 1:
        click.secho('setup.py not found under ' + ('current working directory' if package_path is None else os.path.dirname(package_path)),
                    fg='red',
                    err=True)
        ctx.exit(1)
        return

    if package_path is None:
        package_path = './' + module_name        
    
    print('Use dong ML project: ' + module_name)
    print('Project path: ' + os.path.abspath(os.path.dirname(package_path)))

    (login, password) = auth.get_credential_or_exit()
    _execute(password, package_path, module_name, message, json.dumps(list(args)))


@command.command()
@click.option('-j', '--job-name', required=True)
def status(job_name):
    """Retrieve training status, training message note."""

    (login, password) = auth.get_credential_or_exit()
    headers = net.authorization_headers(password)
    try:
        r = httpclient.get('api/v1/train/{}/'.format(job_name), headers=headers)
        result = r.json()
        output = """\njob-name: {}
message: {}
status: {}\n""".format(result['job_name'], result['message'], result['status'])
    except Exception as e:
        output = "Can't get any status of training job: {}, stopped.".format(job_name)

    print(output)


@command.command()
@click.option('-j', '--job-name', required = True)
def kill(job_name):
    """kill running job."""
    (login, password) = auth.get_credential_or_exit()
    headers = net.authorization_headers(password)
    try:
        r = httpclient.post('api/v1/train/{}/kill'.format(job_name), headers=headers)
        result = r.json()
        output = "\nkilling request has been sent, please wait a moment...\n" + usage_tip.format(job_name)

#        output = """name: {}
#status: {}""".format(result['name'], result['status'])
    except Exception as e:
        output = "fail to kill {}\n{}".format(job_name,e)
    print(output)



import collections

def request_handler(func):
    func.is_request_handler = True
    return func

def get_request_handlers(cls, name_only=False):
    request_handlers = []
    for name in cls.__dict__.keys():
        if hasattr(cls.__dict__[name], "is_request_handler"):
            if name_only:
                request_handlers.append(name)
            else:
                request_handlers.append((name, cls.__dict__[name]))
    return request_handlers


class HyperParameterConfig():

    def __init__(self, config=None, callback_generate_next_config_list=None):
        self._config = config
        self._callback = callback_generate_next_config_list
        
    def get_config(self):
        return self._config

    def get_next_config(self, train_result_dict):
        if self._callback is None:
            return None
        else:
            return self._callback(self._config, train_result_dict)

class HyperParameterTuner():

    def __init__(self):
        pass

    def get_hyper_parameter_config_list(self):
        return None
        
    def get_key_func_for_max_scores(self):
        return None
        
class Service():

    def __init__(self):
        pass


def make_func_args(args_tuple):
    args = []
    kwargs = {}
    i = 0
    while i < len(args_tuple):
        if args_tuple[i][0] == ':' and i is not len(args_tuple):
            kwargs[args_tuple[i][1:]] = args_tuple[i+1]
            i += 1
        else:
            args.append(args_tuple[i])
        i += 1
    return args, kwargs
        
def recur_getattr(obj, obj_name_list):
    if len(obj_name_list) == 0:
        return None
    attr = getattr(obj, obj_name_list[0])
    if len(obj_name_list) == 1:
        return attr
    return recur_getattr(attr, obj_name_list[1])

class Model():

    def __init__(self):
        pass

    # multiple update function?
    def update(self):
        pass
    
    def train(self):
        pass

    def test(self):
        pass
    
    def commit(self):
        pass


    def run_ordered_dict_as_procedure(self, ordered_dict, more_args_dict={}):

        for key  in ordered_dict:
            func_name = key
            args_tuple = ordered_dict[key]
            args, kwargs = make_func_args(args_tuple)
            try:
                if type(more_args_dict[func_name]) == dict:
                    kwargs = {**more_args_dict[func_name], **kwargs}
                elif type(more_args_dict[func_name]) == list:
                    args = args + more_args_dict[func_name]
            except KeyError:
                pass
            obj_name_list = func_name.split('.')
            obj = eval(obj_name_list[0])
            if len(obj_name_list) >= 2:
                func = recur_getattr(obj, obj_name_list[1:])
            else:
                func = obj
            func(*args, **kwargs)

class Data():

    def __init__(self):
        pass

        




import os.path
import string
from pkgutil import get_data

def get_template(name):
    pkg_name = __name__.split(".", 1)[0]
    file_name = "{name}.template".format(name=name)
    data = get_data(pkg_name, os.path.join("templates", file_name))
    return string.Template(data.decode(encoding='utf8'))

def make_template(name, opts):
    template = get_template(name)
    return template.safe_substitute(opts)



import os
import click

import dong.utils as utils
import dong.structure as structure

def init_project(project=None):
    if project:
        utils.create_directory(project)
        click.secho('Generate ML project:' + project + ' successful!', fg='green')

    opts = {}
    if not project:
        cwd = os.getcwd()
        opts['project'] = cwd.split(os.sep)[-1]
    else:
        opts['project'] = project
    opts['package'] = opts['project']

    struct = structure.define_structure(opts)
    structure.create_structure(struct, opts, project)

    # Cannot use new under the project folder
    # File already exists (same name) message should be beauty


from setuptools import setup, find_packages
import os

def read_file(filename):
    filepath = os.path.join(
        os.path.dirname(os.path.dirname(__file__)), filename)
    if os.path.exists(filepath):
        return open(filepath).read()
    else:
        return ''

setup(
    name='dong',
    version='0.2.1.6',
    description='Universal Command Line Interface for Libgirl AI Platform',
    long_description=read_file('readme.md'),
    long_description_content_type="text/markdown",
    url='https://dong.libgirl.com/',
    author='Team Libgirl',
    author_email='team@libgirl.com',
    license='Apache License 2.0',
    packages=find_packages(),
    install_requires=['requests==2.22.0', 'tinynetrc==1.3.0', 'click==7.0'],
    include_package_data=True,
    zip_safe=False,
    test_suite='tests',
    setup_requires=["pytest-runner"],
    tests_require=["pytest"],
    entry_points='''
        [console_scripts]
        dong=dong.cliapp:main
    '''
)



import click

from dong import auth
from dong import net
from dong import httpclient


usage_tip = "\n[Usage] Status check:\ndong endpoint status -e {}\n"

def _bring_up(password, job_name):
    headers = net.authorization_headers(password)
    r = httpclient.post('api/v1/endpoint/', json={'job': job_name}, headers=headers)
    if r.status_code != 200:
        return None
    result = r.json()
    return result['endpoint_name']


@click.group()
def command():
    """Operate on endpoint."""
    pass


@command.command()
@click.argument('job_name')
def up(job_name):
    """Bring up endpoint to serve."""
    (login, password) = auth.get_credential_or_exit()

    print('Bring up...')
    endpoint = _bring_up(password, job_name)
    if endpoint is None:
        print("Can't bring up endpoint, stopped.")
        return
    click.echo(
        "\n"
        + "New endpoint-name: " 
        + click.style(" {} ".format(endpoint), reverse=True) 
        + "\n"
        + usage_tip.format(endpoint)
        )


@command.command()
@click.option('-e', '--endpoint-name', required=True)
def status(endpoint_name):
    """Retrieve endpoint status."""

    (login, password) = auth.get_credential_or_exit()
    headers = net.authorization_headers(password)
    try:
        r = httpclient.get('api/v1/endpoint/{}/'.format(endpoint_name), headers=headers)
        result = r.json()

        click.echo('\nEndpoint name: ' + click.style(result['name'], fg='green'))
        click.echo('External ip: ' + click.style(result['external_ip'], fg='green'))
        click.echo('Status: ' + click.style(result['status'] + "\n"))
    except Exception as e:
        print("\nCan't get any status of endpoint: {}, stopped.\n".format(endpoint_name))


@command.command()
@click.option('-e', '--endpoint-name', required=True)
def kill(endpoint_name):
    """kill running endpoint."""
   
    (login, password) = auth.get_credential_or_exit()
    headers = net.authorization_headers(password)
    try:
        r = httpclient.post('api/v1/endpoint/{}/kill'.format(endpoint_name), headers=headers)
        result = r.json()
        print("{} ready to kill".format(endpoint_name))
        print(usage_tip.format(endpoint_name))  

    except Exception as e:
        print("fail to kill {}\n{}".format(endpoint_name),e)
